% =========================================================================
%   Praktikum MATLAB/Simulink II
%   Technische Universitt Darmstadt
%   IAT, FG rtm (Prof. Dr.-Ing. U. Konigorski)
%
%   Copyright (c) 2017, FG Regelungstechnik und Mechatronik,
%                       Technische Universitt Darmstadt
%   All rights reserved.
%
%   Redistribution and use in source and binary forms, with or withou
%   modification, are permitted provided that the following conditions are
%   met:
%
%   1. Redistributions of source code must retain the above copyright
%      notice, this list of conditions and the following disclaimer.
%
%   2. Redistributions in binary form must reproduce the above copyright
%      notice, this list of conditions and the following disclaimer in the
%      documentation and/or other materials provided with the distribution.
%
%
%   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
%   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
%   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
%   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
%   HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
%   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
%   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
%   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
%   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
%   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
%   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

% =========================================================================

function vPdot = RiccatiDGL(t, vP, stPendel, stTraj, Q, R)
	
	persistent s_stPendel s_stTraj s_Q s_Rinv;
	
	if (nargin > 2)
		s_stPendel = stPendel;
		s_stTraj = stTraj;
		s_Q = Q;
		s_Rinv = inv(R);
		
		vPdot = [];
		
		return;
	end

	% Aus Zeit Arbeitspunkt berechnen
	vX0 = interp1(s_stTraj.vT, s_stTraj.mX.', t);
	vX0 = vX0.';

	u0 = interp1(s_stTraj.vT, s_stTraj.vU.', t);
	
	% Um Arbeitspunkt linearisieren
	[A, B] = linPendelZR2(s_stPendel, vX0, u0);

	n = length(A);
	
	P = reshape(vP, n, n);
	
	Pdot = P * B * s_Rinv * B.' * P - P * A - A.' * P - s_Q;
	
	vPdot = reshape(Pdot, n*n, 1);

end